window.Gleam.initSearch([{"doc":"nessie_cluster","title":"nessie_cluster","content":"# nessie_cluster\n\n[![Package Version](https://img.shields.io/hexpm/v/nessie_cluster)](https://hex.pm/packages/nessie_cluster)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/nessie_cluster/)\n\nSmall but mighty: a library for forming Erlang clusters in Gleam via DNS.\n\nThis is a port of the [Elixir `DNSCluster` library](https://hex.pm/packages/dns_cluster)\nto Gleam. (Note this project does not have any affiliation with the Elixir library)\n\n## Examples\n\n### Fly.io\n\nSee the [example repo](https://github.com/ckreiling/nessie_cluster_example) for an example\nGleam web service with clustering on Fly.io.\n\n### Unsupervised\n\nThis is **NOT** recommended. [See below](#Supervised) for a supervised example.\n\n```sh\ngleam add nessie_cluster\ngleam add gleam_erlang # for sleeping\n```\n```gleam\nimport gleam/option.{None}\nimport gleam/erlang/process\nimport nessie_cluster\n\npub fn main_simple() {\n    let cluster: nessie_cluster.DnsCluster = nessie_cluster.with_query(\n      nessie_cluster.new(), \n      nessie_cluster.DnsQuery(&quot;api.internal&quot;)\n    )\n\n    // The process queries DNS periodically, connecting to nodes\n    let assert Ok(_subject) = nessie_cluster.start_spec(cluster, None)\n\n    process.sleep_forever()\n}\n```\n\n### Supervised\n\n**It is strongly recommended to start the process under a supervisor,\nensuring it is restarted if it crashes.**\n\nThis example also sources the DNS name from an environment variable.\n\n```sh\ngleam add nessie_cluster\ngleam add gleam_erlang\ngleam add gleam_otp\n```\n\n```gleam\nimport gleam/option.{None}\nimport gleam/erlang/os\nimport gleam/erlang/process\nimport gleam/otp/supervisor\nimport nessie_cluster\n\npub fn main() {\n    // Source the DNS name from an env var\n    let dns_query = case os.get_env(&quot;DISCOVERY_DNS_NAME&quot;) {\n        // DNS queries will occur periodically for the given DNS name\n        Ok(dns_name) -&gt; nessie_cluster.DnsQuery(dns_name)\n        // ensures DNS queries never occur, e.g. for local development\n        Error(Nil) -&gt; nessie_cluster.Ignore\n    }\n\n    let cluster: nessie_cluster.DnsCluster =\n        nessie_cluster.with_query(nessie_cluster.new(), dns_query)\n\n    let cluster_worker =\n        supervisor.worker(fn(_) { \n            nessie_cluster.start_spec(cluster, None)\n        })\n\n    let children = fn(children) {\n        children\n        |&gt; supervisor.add(cluster_worker)\n        // add other children to your supervisor, e.g. a web server ...\n    }\n  \n    let assert Ok(_) = supervisor.start(children)\n  \n    process.sleep_forever()\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/nessie_cluster&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"nessie_cluster","title":"DnsCluster","content":"pub opaque type DnsCluster\n The central DNS cluster type.\n\n Use the `new` function to create a new cluster, and then\n use the `with_*` functions to configure it.\n\nSynonyms:\nnessie_cluster.DnsCluster\nnessie_cluster DnsCluster","url":"nessie_cluster.html#DnsCluster"},{"doc":"nessie_cluster","title":"DnsQuery","content":"pub type DnsQuery {\n  DnsQuery(query: String)\n  Ignore\n}\n A DNS query to be used to discover ndoes.\nDnsQuery(query: String)\n A DNS name to query.\n\nIgnore\n Indicate that the cluster should not perform DNS lookups,\n and therefore not connect to any nodes.\n\n This is useful when running in a development environment\n where clustering isn&#39;t necessary.\n\nSynonyms:\nnessie_cluster.DnsQuery\nnessie_cluster DnsQuery","url":"nessie_cluster.html#DnsQuery"},{"doc":"nessie_cluster","title":"Logger","content":"pub type Logger =\n  fn(LoggerLevel, String) -&gt; Nil\n A simple logger type which should print the message.\n\nSynonyms:\nnessie_cluster.Logger\nnessie_cluster Logger","url":"nessie_cluster.html#Logger"},{"doc":"nessie_cluster","title":"LoggerLevel","content":"pub type LoggerLevel =\n  String\n\n\nSynonyms:\nnessie_cluster.LoggerLevel\nnessie_cluster LoggerLevel","url":"nessie_cluster.html#LoggerLevel"},{"doc":"nessie_cluster","title":"Message","content":"pub opaque type Message\n\n\nSynonyms:\nnessie_cluster.Message\nnessie_cluster Message","url":"nessie_cluster.html#Message"},{"doc":"nessie_cluster","title":"NodeConnectError","content":"pub type NodeConnectError {\n  NodeConnectError(node: Atom, error: ConnectError)\n}\n\nNodeConnectError(node: Atom, error: ConnectError)\n\nnode\n&lt;p&gt;The &lt;code&gt;nodename@nodeip&lt;/code&gt; atom that failed to connect.&lt;/p&gt;\n\nerror\n&lt;p&gt;The error that occurred while trying to connect.&lt;/p&gt;\n\nSynonyms:\nnessie_cluster.NodeConnectError\nnessie_cluster NodeConnectError","url":"nessie_cluster.html#NodeConnectError"},{"doc":"nessie_cluster","title":"Resolver","content":"pub type Resolver {\n  Resolver(\n    basename: fn(Atom) -&gt; Result(String, Nil),\n    connect_node: fn(Atom) -&gt; Result(Node, ConnectError),\n    list_nodes: fn() -&gt; List(Node),\n    lookup: fn(String) -&gt; List(String),\n  )\n}\n The engine powering DNS lookups and node connections. This is\n mostly useful for testing; the Resolver returned by\n `default_resolver` should be sufficient for most - if not all - uses.\nResolver(\n  basename: fn(Atom) -&gt; Result(String, Nil),\n  connect_node: fn(Atom) -&gt; Result(Node, ConnectError),\n  list_nodes: fn() -&gt; List(Node),\n  lookup: fn(String) -&gt; List(String),\n)\n\nbasename\n&lt;p&gt;Extract the base name of a node (i.e. everything before the &lt;code&gt;@&lt;/code&gt; symbol).&lt;/p&gt;\n\nconnect_node\n&lt;p&gt;Connect to a node by name.&lt;/p&gt;\n\nlist_nodes\n&lt;p&gt;List all visible nodes.&lt;/p&gt;\n\nlookup\n&lt;p&gt;Perform a DNS lookup, returning a list of IP addresses\nas strings.&lt;/p&gt;\n\nSynonyms:\nnessie_cluster.Resolver\nnessie_cluster Resolver","url":"nessie_cluster.html#Resolver"},{"doc":"nessie_cluster","title":"default_resolver","content":"pub fn default_resolver() -&gt; Resolver\n Returns the default resolver which will query for A and AAAA\n records, and try to connect to Erlang nodes at all of the\n returned IP addresses.\n\n Most library consumers will never use this function, however it is provided\n in-case a more complex resolution strategy is desired.\nSynonyms:\nnessie_cluster.default_resolver\nnessie_cluster default_resolver","url":"nessie_cluster.html#default_resolver"},{"doc":"nessie_cluster","title":"discover_nodes","content":"pub fn discover_nodes(\n  on subject: Subject(Message),\n  timeout_millis timeout: Option(Int),\n) -&gt; Result(\n  #(List(Node), List(NodeConnectError)),\n  CallError(#(List(Node), List(NodeConnectError))),\n)\n Triggers node discovery manually, returning a tuple containing the list of\n connected nodes (after doing discovery) and a list of connection errors that\n occurred, if any.\n\n If no timeout is supplied, this function returns immediately with `Ok(#([], []))`\n after sending a message to trigger node discovery.\n\n This function is only useful for advanced cases - the DNS cluster\n polls at a regular interval by default, which is typically sufficient.\nSynonyms:\nnessie_cluster.discover_nodes\nnessie_cluster discover_nodes","url":"nessie_cluster.html#discover_nodes"},{"doc":"nessie_cluster","title":"has_ran","content":"pub fn has_ran(\n  subject: Subject(Message),\n  timeout: Int,\n) -&gt; Result(Bool, CallError(Bool))\n Returns a boolean indicating whether DNS discovery has\n ran at least once.\n\n Useful for running in a healthcheck to ensure at least 1\n DNS discovery cycle has ran.\nSynonyms:\nnessie_cluster.has_ran\nnessie_cluster has_ran","url":"nessie_cluster.html#has_ran"},{"doc":"nessie_cluster","title":"new","content":"pub fn new() -&gt; DnsCluster\n Create a new DnsCluster for the given query.\n\n Note that the returned DNS cluster does not actually\n do anything if a query is not provided using\n `with_query`.\nSynonyms:\nnessie_cluster.new\nnessie_cluster new","url":"nessie_cluster.html#new"},{"doc":"nessie_cluster","title":"start_spec","content":"pub fn start_spec(\n  cluster: DnsCluster,\n  parent_subject: Option(Subject(Subject(Message))),\n) -&gt; Result(Subject(Message), StartError)\n Starts an actor which will periodically poll DNS for\n IP addresses and connect to Erlang nodes.\n\n If the cluster&#39;s query is `Ignore`, the actor will start\n successfully, but will not perform any DNS lookups or\n attempt to connect to any nodes.\nSynonyms:\nnessie_cluster.start_spec\nnessie_cluster start_spec","url":"nessie_cluster.html#start_spec"},{"doc":"nessie_cluster","title":"stop","content":"pub fn stop(\n  subject: Subject(Message),\n  timeout: Int,\n) -&gt; Result(Nil, CallError(Nil))\n Stop the DNS cluster.\n\n This causes the actor to exit normally.\nSynonyms:\nnessie_cluster.stop\nnessie_cluster stop","url":"nessie_cluster.html#stop"},{"doc":"nessie_cluster","title":"with_interval","content":"pub fn with_interval(\n  for cluster: DnsCluster,\n  millis interval: Option(Int),\n) -&gt; DnsCluster\n Set the interval at which DNS is polled. If `None`, the\n cluster will only poll DNS when `discover_nodes` is called.\n\n Defaults to 5000 milliseconds.\nSynonyms:\nnessie_cluster.with_interval\nnessie_cluster with_interval","url":"nessie_cluster.html#with_interval"},{"doc":"nessie_cluster","title":"with_logger","content":"pub fn with_logger(\n  for cluster: DnsCluster,\n  using logger: fn(String, String) -&gt; Nil,\n) -&gt; DnsCluster\n Turn logging on or off for the given cluster.\n\n Defaults to a `gleam/io.println()` call, prepending the log level\n to the message.\nSynonyms:\nnessie_cluster.with_logger\nnessie_cluster with_logger","url":"nessie_cluster.html#with_logger"},{"doc":"nessie_cluster","title":"with_name","content":"pub fn with_name(\n  for cluster: DnsCluster,\n  using name: Atom,\n) -&gt; DnsCluster\n Use a custom name for the process.\n\n The default `nessie_cluster` name is typically sufficient.\nSynonyms:\nnessie_cluster.with_name\nnessie_cluster with_name","url":"nessie_cluster.html#with_name"},{"doc":"nessie_cluster","title":"with_query","content":"pub fn with_query(\n  for cluster: DnsCluster,\n  using q: DnsQuery,\n) -&gt; DnsCluster\n Set the query for the given DNS cluster.\n\n Defaults to `Ignore`.\nSynonyms:\nnessie_cluster.with_query\nnessie_cluster with_query","url":"nessie_cluster.html#with_query"},{"doc":"nessie_cluster","title":"with_resolver","content":"pub fn with_resolver(\n  for cluster: DnsCluster,\n  using resolver: Resolver,\n) -&gt; DnsCluster\n Set a custom resolver for the given cluster.\n\n For details on the default resolver, see the documentation for the `default_resolver`\n function.\n\n Most users will not need to change this, but\n it can be useful for testing or more advanced\n use-cases.\nSynonyms:\nnessie_cluster.with_resolver\nnessie_cluster with_resolver","url":"nessie_cluster.html#with_resolver"},{"doc":"nessie_cluster","title":"nessie_cluster","content":" An actor that periodically polls DNS for IP addresses and connects to Erlang nodes.\n\n A DNS query is made every `interval` milliseconds to discover new IPs.\n Nodes will only be joined if their basename matches the basename of the current node.\n Nodes running on remote hosts but with different basenames will fail to connect and\n be ignored.\n\n For all configuration options, see the `with_*` functions in this module.","url":"nessie_cluster.html"}]);